/* -*- C++ -*- */
// $Id: File_Parser.h,v 4.9 1998/01/04 18:14:58 schmidt Exp $

// ============================================================================
//
// = LIBRARY
//    gateway
// 
// = FILENAME
//    File_Parser.h
//
// = AUTHOR
//    Doug Schmidt 
// 
// ============================================================================

#if !defined (_FILE_PARSER)
#define _FILE_PARSER

#include "ace/OS.h"

class FP 
{
  // = TITLE
  //     This class serves as a namespace for the <Return_Type>.
public:
  enum Return_Type 
  { 
    EOLINE,
    EOFILE,
    SUCCESS, 
    COMMENT, 
    DEFAULT,
    PARSE_ERROR
  };
};

template <class ENTRY>
class File_Parser
{
  // = TITLE
  //     Class used to parse the configuration file for the
  //     <Consumer_Map>.
public:
  // = Open and Close the file specified
  int open (const char filename[]);
  int close (void);

  virtual FP::Return_Type read_entry (ENTRY &entry,
                                      int &line_number) = 0;
  // Pure virtual hook that subclasses override and use the protected
  // methods to fill in the <entry>.

protected:
  FP::Return_Type getword (char buf[]);
  // Read the next ASCII word.

  FP::Return_Type getint (ACE_INT32 &value);
  // Read the next integer.

  FP::Return_Type readword (char buf[]);
  // Read the next "word," which is demarcated by <delimiter>s.  
  //
  // @@ This function is inherently flawed since it doesn't take a
  // count of the size of <buf>...

  int delimiter (char ch);
  // Returns true if <ch> is a delimiter, i.e., ' ', ',', or '\t'.

  int comments (char ch);
  // Returns true if <ch> is the comment character, i.e., '#'.

  int skipline (void);
  // Skips to the remainder of a line, e.g., when we find a comment
  // character.

  FILE *infile_;
  // Pointer to the file we're reading.
};

#if defined (ACE_TEMPLATES_REQUIRE_SOURCE)
#include "File_Parser.cpp"
#endif /* ACE_TEMPLATES_REQUIRE_SOURCE */

#if defined (ACE_TEMPLATES_REQUIRE_PRAGMA)
#pragma implementation ("File_Parser.cpp")
#endif /* ACE_TEMPLATES_REQUIRE_PRAGMA */

#endif /* _FILE_PARSER */
